@@ -1,5 +1,5 @@
 package Test::Inter;
-# Copyright (c) 2010-2014 Sullivan Beck. All rights reserved.
+# Copyright (c) 2010-2019 Sullivan Beck. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -11,9 +11,10 @@ use warnings;
 use strict;
 use File::Basename;
 use IO::File;
+use Cwd 'abs_path';
 
 our($VERSION);
-$VERSION = '1.06';
+$VERSION = '1.10';
 
 ###############################################################################
 # BASE METHODS
@@ -51,6 +52,8 @@ sub new {
                'mode'     => 'test', # mode to run script in
                'width'    => 80,     # width of terminal
                'features' => {},     # a list of available features
+               'use_lib'  => 'on',   # whether to run 'use lib' when loading
+                                     # this module
 
                'skipall'  => '',     # the reason for skipping all
                                      # remaining tests
@@ -99,39 +102,60 @@ sub new {
       print "\nRunning $name...\n";
    }
 
-   # Find the test directory
-   #
-   # Scripts will either be run:
-   #    directly (look at $0)
-   #    as a test suite (look for ./t and ../t)
+   # We assume that the module is distributed in a directory with the correct
+   # hierarchy.  This is:
+   #      /some/path      MODDIR
+   #                /t    TESTDIR
+   #                /lib  LIBDIR
+   # We'll find the full path to each.
+
+   my($moddir,$testdir,$libdir);
 
-   my($TDIR,@LIBDIR);
    if (-f "$0") {
-      my $COM = $0;
-      $TDIR   = dirname($COM);
-      $TDIR   = '.'  if (! $TDIR);
-   } elsif (-d 't') {
-      $TDIR   = 't';
-   } else {
-      $TDIR   = '.';
+      $moddir = dirname(dirname(abs_path($0)));
+   } elsif (-d "./t") {
+      $moddir = dirname(abs_path('.'));
+   } elsif (-d "../t") {
+      $moddir = dirname(abs_path('..'));
+   }
+   if (-d "$moddir/t") {
+      $testdir = "$moddir/t";
+   }
+   if (-d "$moddir/lib") {
+      $libdir  = "$moddir/lib";
    }
 
-   @LIBDIR = ('.');
-   if (-d "$TDIR/lib") {
-      push(@LIBDIR,"$TDIR/lib");
-   }
-   if (-d "$TDIR/../lib") {
-      push(@LIBDIR,"$TDIR/../lib");
-   }
+   $$self{'moddir'}  = $moddir;
+   $$self{'testdir'} = $testdir;
+   $$self{'libdir'}  = $libdir;
 
-   $$self{'testdir'} = $TDIR;
-   $$self{'libdir'}  = \@LIBDIR;
+   $self->use_lib();
 
    return $self;
 }
 
+sub use_lib {
+   my($self,$val) = @_;
+   if (defined $val) {
+      $$self{'use_lib'} = $val;
+      return;
+   }
+
+   if ($$self{'use_lib'} eq 'on') {
+      foreach my $dir ($$self{'libdir'},$$self{'testdir'}) {
+         next  if (! defined $dir);
+         eval "use lib '$dir'";
+      }
+   }
+}
+
 sub testdir {
-   my($self) = @_;
+   my($self,$req) = @_;
+   if ($req  &&  $req eq 'mod') {
+      return $$self{'moddir'};
+   } elsif ($req  &&  $req eq 'lib') {
+      return $$self{'libdir'};
+   }
    return $$self{'testdir'};
 }
 
@@ -283,7 +307,8 @@ sub require_ok {
    $main::TI_NUM++  unless ($mode eq 'feature');
 
    my $pack = caller;
-   my @inc  = map { "unshift(\@INC,'$_');\n" } @{ $$self{'libdir'} };
+   my @inc  = map { "unshift(\@INC,'$_');\n" }
+     ($$self{'libdir'},$$self{'testdir'});
 
    my($desc,$code);
 
@@ -297,10 +322,14 @@ REQUIRE
    } else {
       $module = qq['$module'] unless $self->_is_module_name($module);
       $desc   = "require $module";
+      # So the following do not get picked up by cpantorpm-depreq or other
+      # tools that try to determine dependencies and prereqs
+      my $p   = "package";
+      my $r   = "require";
       $code   = <<REQUIRE;
-package $pack;
+$p $pack;
 @inc
-require $module;
+$r $module;
 1;
 REQUIRE
    }
@@ -398,10 +427,13 @@ USE
       my $imports = (@args ? 'qw(' . join(' ',@args) . ')' : '');
       $desc = "use $module $vers $imports";
 
-      my @inc  = map { "unshift(\@INC,'$_');\n" } @{ $$self{'libdir'} };
+      my @inc  = map { "unshift(\@INC,'$_');\n" } ($$self{'libdir'},$$self{'testdir'});
 
+      # So the following do not get picked up by cpantorpm-depreq or other
+      # tools that try to determine dependencies and prereqs
+      my $p   = "package";
       $code = <<USE;
-package $pack;
+$p $pack;
 @inc
 use $module $vers $imports;
 1;
@@ -940,13 +972,15 @@ sub tests {
    if (exists $opts{'feature'}) {
       foreach my $feature (@{ $opts{'feature'} }) {
          $skip = "Required feature unavailable: $feature", last
-           if (! exists $$self{'features'}{$feature});
+           if (! exists $$self{'features'}{$feature}  ||
+               ! $$self{'features'}{$feature});
       }
    }
    if (exists $opts{'disable'}  &&  ! $skip) {
       foreach my $feature (@{ $opts{'disable'} }) {
          $skip = "Disabled due to feature being available: $feature", last
-           if (exists $$self{'features'}{$feature});
+           if (exists $$self{'features'}{$feature}  &&
+               $$self{'features'}{$feature});
       }
    }
 
@@ -1413,24 +1447,42 @@ sub _skip {
          # Check for quoted
 
          ($err,$found,$match,@val) = $self->_parse_quoted($test,$EOT);
-         last  if ($found  ||  $err);
+         last  if ($err);
+         if ($found) {
+            # ''  remains ''
+            last;
+         }
 
          # Check for open
 
          ($err,$found,$match,@val) = $self->_parse_open_close($test,$EOT,$lparen,')');
-         last  if ($found  ||  $err);
+         last  if ($err);
+         if ($found) {
+            # ()  is an empty list
+            if (@val == 1  &&  $val[0] eq '') {
+               @val = ();
+            }
+            last;
+         }
 
          ($err,$found,$match,@val) = $self->_parse_open_close($test,$EOT,$lbrack,']');
          last  if ($err);
          if ($found) {
-            @val = ( [@val] );
+            # []  is []
+            if (@val == 1  &&  $val[0] eq '') {
+               @val = ([]);
+            } else {
+               @val = ( [@val] );
+            }
             last;
          }
 
          ($err,$found,$match,@val) = $self->_parse_open_close($test,$EOT,$lbrace,'}');
          last  if ($err);
          if ($found) {
-            if (@val % 2 == 0) {
+            if (@val == 1  &&  $val[0] eq '') {
+               @val = ( {} );
+            } elsif (@val % 2 == 0) {
                # Even number of elements
                @val = ( {@val} );
             } elsif (! defined $val[$#val]  ||
